@@ -31,18 +31,80 @@ def green(text: str) -> str:
     return color(text, "green")
 
 
-def print_header(text: str, line_width: int = LINE_WIDTH) -> None:
-    header = f" {text} ".center(line_width, "=")
-    logger.info(f"\n{header}\n")
+def yellow(text: str) -> str:
+    return color(text, "yellow")
+
+
+def print_header(
+    text: str,
+    line_width: int = LINE_WIDTH,
+    char: str = "=",
+    leading_newline: bool = True,
+) -> None:
+    header = f" {text} ".center(line_width, char)
+    if leading_newline:
+        header = "\n" + header
+    logger.info(f"{header}\n")
+
+
+def print_content_error(
+    model: str,
+    explore: str,
+    message: str,
+    content_type: str,
+    tile_type: Optional[str],
+    tile_title: Optional[str],
+    space: str,
+    title: str,
+    url: str,
+):
+    path = f"{title} [{space}]"
+    print_header(red(path), LINE_WIDTH + COLOR_CODE_LENGTH)
+
+    if content_type == "dashboard":
+        if tile_type == "dashboard_filter":
+            tile_type = "Filter"
+        else:
+            tile_type = "Tile"
+        line = f"{tile_type} '{tile_title}' failed validation."
+        wrapped = textwrap.fill(line, LINE_WIDTH)
+        logger.info(wrapped + "\n")
+
+    line = f"Error in {model}/{explore}: {message}"
+    wrapped = textwrap.fill(line, LINE_WIDTH)
+    logger.info(wrapped)
+
+    content_type = content_type.title()
+    logger.info("\n" + f"{content_type.title()}: {url}")
 
 
 def print_data_test_error(
-    model: str, explore: str, test_name: str, message: str
+    model: str, explore: str, test_name: str, message: str, lookml_url: str
 ) -> None:
     path = f"{model}/{explore}/{test_name}"
     print_header(red(path), LINE_WIDTH + COLOR_CODE_LENGTH)
     wrapped = textwrap.fill(message, LINE_WIDTH)
     logger.info(wrapped)
+    logger.info("\n" + f"LookML: {lookml_url}")
+
+
+def print_lookml_error(
+    file_path: str, line_number: int, severity: str, message: str, lookml_url: str
+) -> None:
+    if file_path is None:
+        file_path = "[File name not given by Looker]"
+    header_color = yellow if severity == "warning" else red
+    print_header(
+        header_color(f"{file_path}:{line_number}"), LINE_WIDTH + COLOR_CODE_LENGTH
+    )
+    wrapped = textwrap.fill(f"{severity.title()}: {message}", LINE_WIDTH)
+    logger.info(wrapped)
+    if lookml_url:
+        logger.info("\n" + f"LookML: {lookml_url}")
+
+
+def print_lookml_success() -> None:
+    logger.info(green("✓ No LookML errors found."))
 
 
 def print_sql_error(
@@ -70,10 +132,14 @@ def print_sql_error(
     logger.info("\n" + f"Test SQL: {file_path}")
 
 
-def print_validation_result(passed: bool, source: str):
-    bullet = "✓" if passed else "✗"
-    message = green(source) if passed else red(source)
-    status = "passed" if passed else "failed"
+def print_validation_result(status: str, source: str):
+    bullet = "✗" if status == "failed" else "✓"
+    if status == "passed":
+        message = green(source)
+    elif status == "failed":
+        message = red(source)
+    elif status == "skipped":
+        message = dim(source)
     logger.info(f"{bullet} {message} {status}")
 
 
